# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['netappfiles'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Resources.
"""

helps['netappfiles account'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Account Resources.
"""

helps['netappfiles account ad'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Account active directories.
"""

helps['netappfiles account ad add'] = """
type: command
short-summary: Add an active directory to the account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
  - name: --username
    short-summary: Username of Active Directory domain administrator
  - name: --password
    short-summary: Plain text password of Active Directory domain administrator
  - name: --domain
    short-summary: Name of the Active Directory domain
  - name: --dns
    short-summary: Comma separated list of DNS server IP addresses for the Active Directory domain
  - name: --smb-server-name
    short-summary: NetBIOS name of the SMB server. This name will be registered as a computer account in the AD and used to mount volumes. Must be 10 characters or less
  - name: --organizational-unit
    short-summary: The Organizational Unit (OU) within the Windows Active Directory
  - name: --site
    short-summary: The Active Directory site the service will limit Domain Controller discovery to
  - name: --kdc-ip
    short-summary: kdc server IP addresses for the active directory machine. This optional parameter is used only while creating kerberos volume
  - name: --ad-name
    short-summary: Name of the active directory machine. This optional parameter is used only while creating kerberos volume
  - name: --server-root-ca-cert
    short-summary: When LDAP over SSL/TLS is enabled, the LDAP client is required to have base64 encoded Active Directory Certificate Service's self-signed root CA certificate, this optional parameter is used only for dual protocol with LDAP user-mapping volumes.
  - name: --backup-operators
    short-summary: Users to be added to the Built-in Backup Operator active directory group. A space separated string of unique usernames without domain specifier
  - name: --aes-encryption
    short-summary: If enabled, AES encryption will be enabled for SMB communication
  - name: --ldap-signing
    short-summary: Specifies whether or not the LDAP traffic needs to be signed
  - name: --security-operators
    short-summary: Domain Users in the Active directory to be given SeSecurityPrivilege privilege (Needed for SMB Continuously available shares for SQL). A space separated list of unique usernames without domain specifier
  - name: --ldap-over-tls
    short-summary: Specifies whether or not the LDAP traffic needs to be secured via TLS
  - name: --allow-local-ldap-users
    short-summary: If enabled, NFS client local users can also (in addition to LDAP users) access the NFS volumes
  - name: --administrators
    short-summary: Users to be added to the Built-in Administrators active directory group. A space separated string of unique usernames without domain specifier.
  - name: --encrypt-dc-conn
    short-summary: If enabled, Traffic between the SMB server to Domain Controller (DC) will be encrypted
  - name: --user-dn
    short-summary: This specifies the user DN, which overrides the base DN for user lookups
  - name: --group-dn
    short-summary: This specifies the group DN, which overrides the base DN for group lookups
  - name: --group-filter
    short-summary: This specifies the custom LDAP search filter to be used when looking up group membership from LDAP server
examples:
  - name: Add an active directory to the account
    text: >
        az netappfiles account ad add -g mygroup --name myname --username aduser --password aduser --smb-server-name SMBSERVER --dns 1.2.3.4 --domain westcentralus
"""

helps['netappfiles account ad update'] = """
type: command
short-summary: Updates an active directory to the account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
  - name: --active-directory-id
    short-summary: The id of the Active Directory
  - name: --username
    short-summary: Username of Active Directory domain administrator
  - name: --password
    short-summary: Plain text password of Active Directory domain administrator
  - name: --domain
    short-summary: Name of the Active Directory domain
  - name: --dns
    short-summary: Comma separated list of DNS server IP addresses for the Active Directory domain
  - name: --smb-server-name
    short-summary: NetBIOS name of the SMB server. This name will be registered as a computer account in the AD and used to mount volumes. Must be 10 characters or less
  - name: --organizational-unit
    short-summary: The Organizational Unit (OU) within the Windows Active Directory
  - name: --site
    short-summary: The Active Directory site the service will limit Domain Controller discovery to.
  - name: --kdc-ip
    short-summary: kdc server IP addresses for the active directory machine. This optional parameter is used only while creating kerberos volume
  - name: --ad-name
    short-summary: Name of the active directory machine. This optional parameter is used only while creating kerberos volume
  - name: --server-root-ca-cert
    short-summary: When LDAP over SSL/TLS is enabled, the LDAP client is required to have base64 encoded Active Directory Certificate Service's self-signed root CA certificate, this optional parameter is used only for dual protocol with LDAP user-mapping volumes.
  - name: --backup-operators
    short-summary: Users to be added to the Built-in Backup Operator active directory group. A space separated list of unique usernames without domain specifier
  - name: --aes-encryption
    short-summary: If enabled, AES encryption will be enabled for SMB communication
  - name: --ldap-signing
    short-summary: Specifies whether or not the LDAP traffic needs to be signed
  - name: --security-operators
    short-summary: Domain Users in the Active directory to be given SeSecurityPrivilege privilege (Needed for SMB Continuously available shares for SQL). A space separated list of unique usernames without domain specifier
  - name: --ldap-over-tls
    short-summary: Specifies whether or not the LDAP traffic needs to be secured via TLS
  - name: --allow-local-ldap-users
    short-summary: If enabled, NFS client local users can also (in addition to LDAP users) access the NFS volumes
  - name: --administrators
    short-summary: Users to be added to the Built-in Administrators active directory group. A space separated list of unique usernames without domain specifier.
  - name: --encrypt-dc-conn
    short-summary: If enabled, Traffic between the SMB server to Domain Controller (DC) will be encrypted
  - name: --user-dn
    short-summary: This specifies the user DN, which overrides the base DN for user lookups.
  - name: --group-dn
    short-summary: This specifies the group DN, which overrides the base DN for group lookups.
  - name: --group-filter
    short-summary: This specifies the custom LDAP search filter to be used when looking up group membership from LDAP server.
examples:
  - name: Update an active directory on the account
    text: >
        az netappfiles account ad update -g mygroup --name myname --active-directory-id 123 --username aduser --password aduser --smb-server-name SMBSERVER --dns 1.2.3.4 --domain westcentralus
"""

helps['netappfiles account ad list'] = """
type: command
short-summary: List the active directories of an account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
examples:
  - name: Add an active directory to the account
    text: >
        az netappfiles account ad list -g mygroup --name myname
"""

helps['netappfiles account ad remove'] = """
type: command
short-summary: Remove an active directory from the account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
  - name: --active-directory
    short-summary: The id of the active directory
examples:
  - name: Remove an active directory from the account
    text: >
        az netappfiles account ad remove -g mygroup --name myname --active-directory 13641da9-c0e9-4b97-84fc-4f8014a93848
"""

helps['netappfiles account ad wait'] = """
type: command
short-summary: Wait for a account ad operation.
"""

helps['netappfiles account create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) account. Note that active directories are added using the subgroup commands.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
examples:
  - name: Create an ANF account
    text: >
        az netappfiles account create -g mygroup --name myname -l location --tags testtag1=mytag1 testtag3=mytagg
"""

helps['netappfiles account delete'] = """
type: command
short-summary: Delete the specified ANF account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
examples:
  - name: Delete an ANF account
    text: >
        az netappfiles account delete -g mygroup --name myname
"""

helps['netappfiles account list'] = """
type: command
short-summary: List ANF accounts by subscription or by resource group name.
examples:
  - name: List ANF accounts within a resource group
    text: >
        az netappfiles account list -g mygroup
"""

helps['netappfiles account show'] = """
type: command
short-summary: Get the specified ANF account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
examples:
  - name: Get an ANF account
    text: >
        az netappfiles account show -g mygroup --name myname
"""

helps['netappfiles account update'] = """
type: command
short-summary: Set/modify the tags for a specified ANF account.
parameters:
  - name: --account-name --name -a -n
    short-summary: The name of the ANF account
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
  - name: --encryption
    short-summary: Encryption settings
examples:
  - name: Update the tags of an ANF account
    text: >
        az netappfiles account update -g mygroup --name myname --tags testtag2=mytagb
"""

helps['netappfiles account wait'] = """
type: command
short-summary: Wait for a account operation.
"""

helps['netappfiles account backup'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Account Backup Resources.
"""

helps['netappfiles account backup list'] = """
type: command
short-summary: Get list of all Azure NetApp Files (ANF) Account Backups.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
examples:
  - name: Get a list of all ANF account backup
    text: >
        az netappfiles account backup list -g mygroup --account-name myaccountname
"""

helps['netappfiles account backup show'] = """
type: command
short-summary: Get Backup for a Netapp Files (ANF) Account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-name
    short-summary: The name of the backup
examples:
  - name: Get a list of all ANF account backup
    text: >
        az netappfiles account backup show -g mygroup --account-name myaccountname --backup-name mybackupname
"""

helps['netappfiles account backup delete'] = """
type: command
short-summary: Delete Backup for a Netapp Files (ANF) Account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-name
    short-summary: The name of the backup
examples:
  - name: Get a list of all ANF account backup
    text: >
        az netappfiles account backup delete -g mygroup --account-name myaccountname --backup-name mybackupname
"""

helps['netappfiles account backup wait'] = """
type: command
short-summary: Wait for a account backup operation.
"""


helps['netappfiles account backup-policy'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Backup Policy Resources.
"""

helps['netappfiles account backup-policy create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) backup policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-policy-name -b
    short-summary: The name of the ANF backup policy
  - name: --location -l
    short-summary: The location of the backup
  - name: --daily-backups -d
    short-summary: Daily backups count to keep
  - name: --weekly-backups -w
    short-summary: Weekly backups count to keep
  - name: --monthly-backups -m
    short-summary: Monthly backups count to keep
  - name: --enabled -e
    short-summary: The property to decide policy is enabled or not
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
examples:
  - name: Create an ANF backup policy
    text: >
        az netappfiles account backup-policy create -g mygroup --account-name myaccountname --backup-policy-name mybackuppolicyname -l westus2 --daily-backups 1 --enabled true
"""

helps['netappfiles account backup-policy delete'] = """
type: command
short-summary: Delete the specified ANF backup policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-policy-name -b
    short-summary: The name of the ANF backup policy
examples:
  - name: Delete an ANF backup policy
    text: >
        az netappfiles account backup-policy delete -g mygroup --account-name myaccname --backup-policy-name mybackuppolicyname
"""

helps['netappfiles account backup-policy list'] = """
type: command
short-summary: List the ANF backup policy for the specified account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
examples:
  - name: List the backup policy for the ANF account
    text: >
        az netappfiles account backup-policy list -g mygroup --account-name myname
"""

helps['netappfiles account backup-policy show'] = """
type: command
short-summary: Get the specified ANF backup policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-policy-name -b
    short-summary: The name of the ANF backup policy
examples:
  - name: Get an ANF backup policy
    text: >
        az netappfiles account backup-policy show -g mygroup --account-name myaccname --backup-policy-name mybackuppolicyname
"""

helps['netappfiles account backup-policy update'] = """
type: command
short-summary: Update the specified ANF backup policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --backup-policy-name -b
    short-summary: The name of the ANF backup policy
  - name: --daily-backups -d
    short-summary: Daily backups count to keep
  - name: --weekly-backups -w
    short-summary: Weekly backups count to keep
  - name: --monthly-backups -m
    short-summary: Monthly backups count to keep
  - name: --enabled -e
    short-summary: The property to decide policy is enabled or not
examples:
  - name: Update specific values for an ANF backup policy
    text: >
        az netappfiles account backup-policy update -g mygroup --account-name myaccountname --backup-policy-name mybackuppolicyname -l westus2 --daily-backups 1 --enabled false
"""

helps['netappfiles account backup-policy wait'] = """
type: command
short-summary: Wait for a account backup-policy operation.
"""

helps['netappfiles account renew-credentials'] = """
type: command
short-summary: Renew identity credentials that are used to authenticate to key vault, for customer-managed key encryption. If encryption.identity.principalId does not match identity.principalId, running this operation will fix it.
examples:
  - name: Renew identity credentials for account
    text: >
        az netappfiles account renew-credentials -g mygroup --account-name myaccname
"""

helps['netappfiles pool'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Pool Resources.
"""

helps['netappfiles pool create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) pool.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --name --pool-name -n -p
    short-summary: The name of the ANF pool
  - name: --size
    short-summary: The size for the ANF pool. Must be an integer number of tebibytes in multiples of 4
  - name: --service-level
    short-summary: The service level for the ANF pool
  - name: --qos-type
    short-summary: The qos type of the ANF pool
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
  - name: --cool-access
    short-summary: If enabled (true) the pool can contain cool Access enabled volumes.
  - name: --encryption-type
    short-summary: Encryption type of the capacity pool, set encryption type for data at rest for this pool and all volumes in it. This value can only be set when creating new pool. Possible values are Single or Double. Default value is Single.
examples:
  - name: Create an ANF pool
    text: >
        az netappfiles pool create -g mygroup --account-name myaccountname --name mypoolname -l westus2 --size 8 --service-level premium
"""

helps['netappfiles pool delete'] = """
type: command
short-summary: Delete the specified ANF pool.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --name --pool-name -n -p
    short-summary: The name of the ANF pool
examples:
  - name: Delete an ANF pool
    text: >
        az netappfiles pool delete -g mygroup --account-name myaccname --name mypoolname
"""

helps['netappfiles pool list'] = """
type: command
short-summary: L:ist the ANF pools for the specified account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
examples:
  - name: List the pools for the ANF account
    text: >
        az netappfiles pool list -g mygroup --account-name myname
"""

helps['netappfiles pool show'] = """
type: command
short-summary: Get the specified ANF pool.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --name --pool-name -n -p
    short-summary: The name of the ANF pool
examples:
  - name: Get an ANF pool
    text: >
        az netappfiles pool show -g mygroup --account-name myaccname --name mypoolname
"""

helps['netappfiles pool update'] = """
type: command
short-summary: Update the tags of the specified ANF pool.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --name --pool-name -n -p
    short-summary: The name of the ANF pool
  - name: --size
    short-summary: The size for the ANF pool. Must be an integer number of tebibytes in multiples of 4
  - name: --qos-type
    short-summary: The qos type of the ANF pool
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
  - name: --cool-access
    short-summary: If enabled (true) the pool can contain cool Access enabled volumes
examples:
  - name: Update specific values for an ANF pool
    text: >
        az netappfiles pool update -g mygroup --account-name myaccname --name mypoolname --tags mytag1=abcd mytag2=efgh
"""

helps['netappfiles pool wait'] = """
type: command
short-summary: Wait for a pool operation.
"""


helps['netappfiles snapshot'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Snapshot Resources.
"""

helps['netappfiles snapshot create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) snapshot.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --name --snapshot-name -n -s
    short-summary: The name of the ANF snapshot
examples:
  - name: Create an ANF snapshot
    text: >
        az netappfiles snapshot create -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --name mysnapname -l eastus
"""

helps['netappfiles snapshot delete'] = """
type: command
short-summary: Delete the specified ANF snapshot.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --name --snapshot-name -n -s
    short-summary: The name of the ANF snapshot
examples:
  - name: Delete an ANF snapshot
    text: >
        az netappfiles snapshot delete -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --name mysnapname
"""

helps['netappfiles snapshot list'] = """
type: command
short-summary: List the snapshots of an ANF volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
examples:
  - name: list the snapshots of an ANF volume
    text: >
        az netappfiles snapshot list -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname
"""

helps['netappfiles snapshot show'] = """
type: command
short-summary: Get the specified ANF snapshot.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --name --snapshot-name -n -s
    short-summary: The name of the ANF snapshot
examples:
  - name: Return the specified ANF snapshot
    text: >
        az netappfiles snapshot show -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --name mysnapname
"""

helps['netappfiles snapshot restore-files'] = """
type: command
short-summary: Restore specified files from the specified snapshot to the active filesystem.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --name --snapshot-name -n -s
    short-summary: The name of the ANF snapshot
  - name: --file-paths
    short-summary: Required. A space separated string of filed to be restored
  - name: --destination-path
    short-summary: Destination folder where the files will be restored
examples:
  - name: Restore files from snapshot
    text: >
        az netappfiles snapshot restore-files -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --name mysnapname --file-paths myfilepaths
"""

helps['netappfiles snapshot wait'] = """
type: command
short-summary: Wait for a snapshot operation.
"""


helps['netappfiles volume'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume Resources.
"""

helps['netappfiles volume create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) volume. Export policies are applied with the subgroup commands but note that volumes are always created with a default export policy
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --service-level
    short-summary: The service level
  - name: --usage-threshold
    short-summary: The maximum storage quota allowed for a file system as integer number of GiB. Min 100 GiB, max 100TiB"
  - name: --file-path
    short-summary: A 1-80 character long alphanumeric string value that identifies a unique file share or mount point in the target delegate subnet
  - name: --vnet
    short-summary: Name or Resource ID of the vnet. If you want to use a vnet in other resource group or subscription, please provide the Resource ID instead of the name of the vnet.
  - name: --subnet
    short-summary: The name of the subnet for the volume.
  - name: --protocol-types
    short-summary: Space separated list of protocols that the volume can use, available protocols are "NFSv4.1", "NFSv3", "CIFS"
  - name: --volume-type
    short-summary: Whether the volume should be a data protection volume ("DataProtection"), empty if this is not a data protection volume
  - name: --endpoint-type
    short-summary: Whether the volume is source ("src") or destination ("dst")
  - name: --remote-volume-resource-id
    short-summary: The volume id of the remote volume of the replication (the destination for "src" volume endpoints and the source for "dst" endpoints)
  - name: --replication-schedule
    short-summary: The replication schedule, e.g. "_10minutely, hourly, daily, weekly, monthly"
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
  - name: --snapshot-id
    short-summary: Create a volume created from this snapshot. UUID v4 or resource identifier used to identify the Snapshot. example snapshot-id "9760acf5-4638-11e7-9bdb-020073ca3333"
  - name: --snapshot-policy-id
    short-summary: Snapshot Policy ResourceId
  - name: --backup-policy-id
    short-summary: Backup Policy Resource ID
  - name: --backup-enabled
    short-summary: Backup Enabled
  - name: --backup-id
    short-summary: Backup ID. UUID v4 or resource identifier used to identify the Backup
  - name: --policy-enforced
    short-summary: Policy Enforced
  - name: --vault-id
    short-summary: Vault Resource ID
  - name: --kerberos-enabled
    short-summary: Describe if a volume is KerberosEnabled
  - name: --throughput-mibps
    short-summary: Maximum throughput in Mibps that can be achieved by this volume
  - name: --snapshot-dir-visible
    short-summary: If enabled (true) the volume will contain a read-only .snapshot directory which provides access to each of the volume's snapshots (default to true).
  - name: --security-style
    short-summary: The security style of volume
  - name: --kerberos5-r
    short-summary: Kerberos5 Read only access
  - name: --kerberos5-rw
    short-summary: Kerberos5 Read and write access
  - name: --kerberos5i-r
    short-summary: Kerberos5i Read only access
  - name: --kerberos5i-rw
    short-summary: Kerberos5i Read and write access
  - name: --kerberos5p-r
    short-summary: Kerberos5p Read only access
  - name: --kerberos5p-rw
    short-summary: Kerberos5p Read and write access
  - name: --has-root-access
    short-summary: Has root access to volume. Possible values are True or False
  - name: --smb-encryption
    short-summary: Enables encryption for in-flight smb3 data. Only applicable for SMB/DualProtocol volume. To be used with swagger version 2020-08-01 or later. Default value is False
  - name: --smb-continuously-avl
    short-summary: Enables continuously available share property for smb volume. Only applicable for SMB volume. Default value is False
  - name: --encryption-key-source
    short-summary: Encryption Key Source
  - name: --allowed-clients
    short-summary: Client ingress specification as comma separated string with IPv4 CIDRs, IPv4 host addresses and host names
  - name: --cifs
    short-summary: Allows CIFS protocol. Enable only for CIFS type volumes
  - name: --rule-index
    short-summary: Order index
  - name: --unix-read-only
    short-summary: Read only access
  - name: --unix-read-write
    short-summary: Read and write access
  - name: --ldap-enabled
    short-summary: Specifies whether LDAP is enabled or not for a given NFS volume
  - name: --chown-mode
    short-summary: This parameter specifies who is authorized to change the ownership of a file. restricted - Only root user can change the ownership of the file. unrestricted - Non-root users can change ownership of files that they own. Possible values include- Restricted, Unrestricted. Default value- Restricted.
  - name: --cool-access
    short-summary: Specifies whether Cool Access(tiering) is enabled for the volume.
  - name: --coolness-period
    short-summary: Specifies the number of days after which data that is not accessed by clients will be tiered.
  - name: --unix-permissions
    short-summary: UNIX permissions for NFS volume accepted in octal 4 digit format. First digit selects the set user ID(4), set group ID (2) and sticky (1) attributes. Second digit selects permission for the owner of the file- read (4), write (2) and execute (1). Third selects permissions for other users in the same group. the fourth for other users not in the group. 0755 - gives read/write/execute permissions to owner and read/execute to group and other users.
  - name: --is-def-quota-enabled
    short-summary: Specifies if default quota is enabled for the volume.
  - name: --default-user-quota
    short-summary: Default user quota for volume in KiBs. If isDefaultQuotaEnabled is set, the minimum value of 4 KiBs applies.
  - name: --default-group-quota
    short-summary: Default group quota for volume in KiBs. If isDefaultQuotaEnabled is set, the minimum value of 4 KiBs applies.
  - name: --avs-data-store
    short-summary: Specifies whether the volume is enabled for Azure VMware Solution (AVS) datastore purpose. Possible values are Enabled and Disabled. Default value is Disabled.
  - name: --network-features
    short-summary: Basic network, or Standard features available to the volume. Possible values are Basic and Standard. Default value is Basic.
  - name: --enable-subvolumes
    short-summary: Flag indicating whether subvolume operations are enabled on the volume. Possible values are Enabled and Disabled. Default value is Disabled
  - name: --zones
    short-summary: Availability Zone
  - name: --kv-private-endpoint-id
    short-summary: The resource ID of private endpoint for KeyVault. It must reside in the same VNET as the volume. Only applicable if encryptionKeySource = 'Microsoft.KeyVault'.
  - name: --kerberos-enabled
    short-summary: Describe if a volume is KerberosEnabled

examples:
  - name: Create an ANF volume
    text: >
        az netappfiles volume create -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname -l westus2 --service-level premium --usage-threshold 100 --file-path "unique-file-path" --vnet myvnet --subnet mysubnet --protocol-types NFSv3 NFSv4.1

  - name: Create an ANF volume with zones (Availability Zone) specified
    text: >
        az netappfiles volume create -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname -l westus2 --service-level premium --usage-threshold 100 --file-path "unique-file-path" --vnet myvnet --subnet mysubnet --protocol-types NFSv3 --zones zone1
"""

helps['netappfiles volume delete'] = """
type: command
short-summary: Delete the specified ANF volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --force-delete
    short-summary: An option to force delete the volume. Will cleanup resources connected to the particular volume.
examples:
  - name: Delete an ANF volume
    text: >
        az netappfiles volume delete -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume revert'] = """
type: command
short-summary: Revert a volume to one of its snapshots.
long-summary: Revert a volume to the snapshot specified in the body.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --snapshot-id -s
    short-summary: SnapshotId of the snapshot. UUID v4 used to identify the Snapshot, example "9760acf5-4638-11e7-9bdb-020073ca3333"
examples:
  - name: Revert a volume to one of its snapshots.
    text: >
        az netappfiles volume revert -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --snapshot-id 9760acf5-4638-11e7-9bdb-020073ca3333
"""

helps['netappfiles volume wait'] = """
type: command
short-summary: Wait for a volume operation.
"""


helps['netappfiles volume replication'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume replication operations.
"""

helps['netappfiles volume replication approve'] = """
type: command
short-summary: Authorize a volume as a replication destination for a specified source.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication source volume
  - name: --remote-volume-resource-id -d
    short-summary: The resource id of the destination replication volume
examples:
  - name: Authorize the volume as the replication destination for the source
    text: >
        az netappfiles volume replication approve -g mygroup --account-name myaccname --pool-name mypoolname --name mysourcevolname --remote-volume-resource-id /subscriptions/69a75bda-882e-44d5-8431-63421204131c/resourceGroups/mygroup1/providers/Microsoft.NetApp/netAppAccounts/myaccount1/capacityPools/mypool1/volumes/mydestinationvolume
"""

helps['netappfiles volume replication suspend'] = """
type: command
short-summary: Suspend/break a volume replication for the specified destination volume. The replication process is suspended until resumed or deleted.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication destination volume
  - name: --force --force-break-replication -f
    short-summary: Force break the replication
examples:
  - name: Suspend the replication process
    text: >
        az netappfiles volume replication suspend -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication resume'] = """
type: command
short-summary: Resync a volume replication for the specified destination volume. The replication process is resumed from source to destination.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication destination volume
examples:
  - name: Resume the replication process
    text: >
        az netappfiles volume replication resume -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication remove'] = """
type: command
short-summary: Delete a volume replication for the specified destination volume. The data replication objects of source and destination volumes will be removed.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication destination volume
examples:
  - name: Delete the replication objects of the paired volumes
    text: >
        az netappfiles volume replication remove -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication re-initialize'] = """
type: command
short-summary: Re-initialise a volume replication for the specified destination volume. The replication process is resumed from source to destination.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication destination volume
examples:
  - name: Re-initialises the replication process
    text: >
        az netappfiles volume replication re-initialize -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication status'] = """
type: command
short-summary: Get the replication status for the specified replication volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the replication destination volume
examples:
  - name: Get the replication status for the volume. Returns whether the replication is healthy, the replication schedule and the mirror state (whether replication is suspened/broken or synced/mirrored)
    text: >
        az netappfiles volume replication status -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication list'] = """
type: command
short-summary: List replication for ANF volume
examples:
  - name: List all replications for a specified volume
    text: >
        az netappfiles volume replication list -g mygroup --account-name myaccname --pool-name mypoolname --name mydestinationvolname
"""

helps['netappfiles volume replication wait'] = """
type: command
short-summary: Wait for a volume replication operation.
"""


helps['netappfiles volume export-policy'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume export policies.
"""

helps['netappfiles volume export-policy add'] = """
type: command
short-summary: Add a new rule to the export policy for a volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --rule-index
    short-summary: Order index. No number can be repeated. Max 6 rules.
  - name: --unix-read-only
    short-summary: Indication of read only access
  - name: --unix-read-write
    short-summary: Indication of read and write access
  - name: --cifs
    short-summary: Indication that CIFS protocol is allowed
  - name: --nfsv3
    short-summary: Indication that NFSv3 protocol is allowed
  - name: --nfsv41
    short-summary: Indication that NFSv4.1 protocol is allowed
  - name: --allowed-clients
    short-summary: Client ingress specification as comma separated string with IPv4 CIDRs, IPv4 host addresses and host names)
  - name: --kerberos5-r
    short-summary: Kerberos5 Read only access
  - name: --kerberos5-rw
    short-summary: Kerberos5 Read and write access
  - name: --kerberos5i-r
    short-summary: Kerberos5i Read only access
  - name: --kerberos5i-rw
    short-summary: Kerberos5i Read and write access
  - name: --kerberos5p-r
    short-summary: Kerberos5p Read only access
  - name: --kerberos5p-rw
    short-summary: Kerberos5p Read and write access
#  - name: --has-root-access
    #short-summary: Has root access to volume. Possible values are True or False
  - name: --chown-mode
    short-summary: This parameter specifies who is authorized to change the ownership of a file. restricted - Only root user can change the ownership of the file. unrestricted - Non-root users can change ownership of files that they own. Possible values include- Restricted, Unrestricted. Default value- Restricted.
examples:
  - name: Add an export policy rule for the ANF volume
    text: >
        az netappfiles volume export-policy add -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --allowed-clients "1.2.3.0/24" --rule-index 2 --unix-read-only true --unix-read-write false --cifs false --nfsv3 true --nfsv41 false
"""

helps['netappfiles volume export-policy list'] = """
type: command
short-summary: List the export policy rules for a volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
examples:
  - name: List the export policy rules for an ANF volume
    text: >
        az netappfiles volume export-policy list -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume export-policy remove'] = """
type: command
short-summary: Remove a rule from the export policy for a volume by rule index. The current rules can be obtained by performing the subgroup list command.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --rule-index
    short-summary: Order index. Range 1 to 6.
examples:
  - name: Remove an export policy rule for an ANF volume
    text: >
        az netappfiles volume export-policy remove -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --rule-index 4
"""

helps['netappfiles volume export-policy wait'] = """
type: command
short-summary: Wait for a volume export-policy operation.
"""


helps['netappfiles volume list'] = """
type: command
short-summary: List the ANF Pools for the specified account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
examples:
  - name: List the ANF volumes of the pool
    text: >
        az netappfiles volume list -g mygroup --account-name myaccname --pool-name mypoolname
"""

helps['netappfiles volume show'] = """
type: command
short-summary: Get the specified ANF volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF pool
examples:
  - name: Returns the properties of the given ANF volume
    text: >
        az netappfiles volume show -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume update'] = """
type: command
short-summary: Update the specified ANF volume with the values provided. Unspecified values will remain unchanged. Export policies are amended/created with the subgroup commands
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --service-level
    short-summary: The service level
  - name: --usage-threshold
    short-summary: The maximum storage quota allowed for a file system as integer number of GiB. Min 100 GiB, max 100TiB
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
  - name: --backup-enabled
    short-summary: Backup Enabled
  - name: --backup-policy-id
    short-summary: Backup Policy Resource ID
  - name: --policy-enforced
    short-summary: Backup Policy Enforced
  - name: --vault-id
    short-summary: Vault Resource ID
  - name: --snapshot-policy-id
    short-summary: Snapshot Policy ResourceId
  - name: --is-def-quota-enabled
    short-summary: Specifies if default quota is enabled for the volume.
  - name: --default-user-quota
    short-summary: Default user quota for volume in KiBs. If isDefaultQuotaEnabled is set, the minimum value of 4 KiBs applies
  - name: --default-group-quota
    short-summary: Default group quota for volume in KiBs. If isDefaultQuotaEnabled is set, the minimum value of 4 KiBs applies
  - name: --throughput-mibps
    short-summary: Maximum throughput in Mibps that can be achieved by this volume and this will be accepted as input only for manual qosType volume
  - name: --unix-permissions
    short-summary: UNIX permissions for NFS volume accepted in octal 4 digit format. First digit selects the set user ID(4), set group ID (2) and sticky (1) attributes. Second digit selects permission for the owner of the file- read (4), write (2) and execute (1). Third selects permissions for other users in the same group. the fourth for other users not in the group. 0755 - gives read/write/execute permissions to owner and read/execute to group and other users.
  - name: --cool-access
    short-summary: If enabled (true) the pool can contain cool Access enabled volumes
  - name: --coolness-period
    short-summary: Specifies the number of days after which data that is not accessed by clients will be tiered.
examples:
  - name: Update an ANF volume
    text: >
        az netappfiles volume update -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --service-level ultra --usage-threshold 100 --tags mytag=specialvol
"""

helps['netappfiles volume pool-change'] = """
type: command
short-summary: Change pool for an Azure NetApp Files (ANF) volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --new-pool-resource-id -d
    short-summary: The resource id of the new ANF pool
examples:
  - name: This changes the pool for the volume myvolname from mypoolname to pool with the Id mynewresourceid
    text: >
        az netappfiles volume pool-change -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --new-pool-resource-id mynewresourceid
"""

helps['netappfiles volume reset-cifs-pw'] = """
type: command
short-summary: Reset CIFS password from an Azure NetApp Files (ANF) volume
examples:
  - name: Reset the CIFS password from volume
    text: >
        az netappfiles volume reset-cifs-pw -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume relocate'] = """
type: command
short-summary: Relocates an Azure NetApp Files (ANF) volume to a new stamp
examples:
  - name: Relocates volume to a new stamp
    text: >
        az netappfiles volume relocate -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume finalize-relocation'] = """
type: command
short-summary: Finalizes the relocation of the Azure NetApp Files (ANF) volume and cleans up the old volume
examples:
  - name: Finalizes the relocation of the volume and cleans up the old volume
    text: >
        az netappfiles volume finalize-relocation -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume revert-relocation'] = """
type: command
short-summary: Reverts the Azure NetApp Files (ANF) volume relocation process, cleans up the new volume and starts using the former-existing volume
examples:
  - name: Reverts the volume relocation process, cleans up the new volume and starts using the former-existing volume
    text: >
        az netappfiles volume revert-relocation -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""


helps['netappfiles volume backup'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume Backup Resources.
"""

helps['netappfiles volume backup create'] = """
type: command
short-summary: Create specified ANF volume backup.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --backup-name -b
    short-summary: The name of the ANF backup
  - name: --use-existing-snapshot
    short-summary: Manual backup an already existing snapshot. This will always be false for scheduled backups and true/false for manual backups
examples:
  - name: Returns the created ANF backup
    text: >
        az netappfiles volume backup create -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname -l westus2 --backup-name mybackupname
"""

helps['netappfiles volume backup list'] = """
type: command
short-summary: List the ANF Backups for the specified volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
examples:
  - name: List the ANF backups of the volume
    text: >
        az netappfiles volume backup list -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume backup show'] = """
type: command
short-summary: Get the specified ANF Backup.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF pool
  - name: --backup-name -b
    short-summary: The name of the ANF backup
examples:
  - name: Returns the properties of the given ANF backup
    text: >
        az netappfiles volume backup show -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --backup-name mybackupname
"""

helps['netappfiles volume backup status'] = """
type: command
short-summary: Get backup status of the specified ANF Volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
examples:
  - name: Returns the backup status of the given ANF Volume
    text: >
        az netappfiles volume backup status -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume backup restore-status'] = """
type: command
short-summary: Get backup restore status of the specified ANF Volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF pool
examples:
  - name: Returns the backup restore status of the given ANF Volume
    text: >
        az netappfiles volume backup restore-status -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname
"""

helps['netappfiles volume backup update'] = """
type: command
short-summary: Update the specified ANF backup with the values provided.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
  - name: --backup-name -b
    short-summary: The name of the ANF backup
  - name: --label
    short-summary: Label for backup.
  - name: --use-existing-snapshot
    short-summary: Manual backup an already existing snapshot. This will always be false for scheduled backups and true or false for manual backups.
examples:
  - name: Update an ANF backup
    text: >
        az netappfiles volume backup update -g mygroup --account-name myaccname --pool-name mypoolname --name myvolname --backup-name mybackupname
"""

helps['netappfiles volume backup wait'] = """
type: command
short-summary: Wait for a volume backup operation.
"""

helps['netappfiles volume quota-rule'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume Quota Rules Resources.
"""

helps['netappfiles volume quota-rule show'] = """
type: command
short-summary: Get ANF Volume Quota Rule for Volume.
examples:
  - name: Get a ANF Volume quote rule
    text: >
        az netappfiles volume quota-rule show -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --quota-rule-name myrulename
"""

helps['netappfiles volume quota-rule create'] = """
type: command
short-summary: Create a ANF Volume Quota Rule.
parameters:
  - name: --tags
    short-summary: Resource tags.
  - name: --quota-rule-name
    short-summary: Name of Quota rule.
  - name: --quota-size
    short-summary: Size of quota.
  - name: --quota-type
    short-summary: Type of quota. Known values are "DefaultUserQuota", "DefaultGroupQuota", "IndividualUserQuota", and "IndividualGroupQuota".
  - name: --quota-target
    short-summary: UserID/GroupID/SID based on the quota target type. UserID and groupID can be found by running ‘id’ or ‘getent’ command for the user or group and SID can be found by running :code:`<wmic useraccount where name='user-name' get sid>`.
examples:
  - name: Create an ANF Volume quote rule
    text: >
        az netappfiles volume quota-rule create -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --quota-rule-name myrulename --quota-type DefaultGroupQuota --quota-size 100006 --quota-target ''
"""


helps['netappfiles volume quota-rule update'] = """
type: command
short-summary: Update a specified ANF Volume Quota Rule.
parameters:
  - name: --quota-size
    short-summary: Size of quota.
  - name: --quota-type
    short-summary: Type of quota. Known values are "DefaultUserQuota", "DefaultGroupQuota", "IndividualUserQuota", and "IndividualGroupQuota".
  - name: --quota-target
    short-summary: UserID/GroupID/SID based on the quota target type. UserID and groupID can be found by running ‘id’ or ‘getent’ command for the user or group and SID can be found by running :code:`<wmic useraccount where name='user-name' get sid>`.
examples:
  - name: Update a specified ANF Volume quote rule
    text: >
        az netappfiles volume quota-rule update -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --quota-rule-name myrulename --quota-size 100006
"""

helps['netappfiles volume quota-rule delete'] = """
type: command
short-summary: Delete a specified ANF Volume Quota Rule for Volume.
examples:
  - name: Delete a specified ANF Volume quote rule
    text: >
        az netappfiles volume quota-rule delete -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname --quota-rule-name myrulename
"""

helps['netappfiles volume quota-rule list'] = """
type: command
short-summary: Get all ANF Volume Quota Rule for specified Volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --name --volume-name -n -v
    short-summary: The name of the ANF volume
examples:
  - name: Get all ANF Volume Quota Rule for specified Volume
    text: >
        az netappfiles volume quota-rule list -g mygroup --account-name myaccname --pool-name mypoolname --volume-name myvolname
"""

helps['netappfiles volume quota-rule wait'] = """
type: command
short-summary: Wait for a volume quota-rule operation.
"""


helps['netappfiles snapshot policy'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Snapshot Policy Resources.
"""

helps['netappfiles snapshot policy create'] = """
type: command
short-summary: Create a new Azure NetApp Files (ANF) snapshot policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --snapshot-policy-name
    short-summary: The name of the ANF snapshot policy
  - name: --hourly-snapshots -u
    short-summary: Hourly snapshots count to keep
  - name: --daily-snapshots -d
    short-summary: Daily snapshots count to keep
  - name: --weekly-snapshots -w
    short-summary: Weekly snapshots count to keep
  - name: --monthly-snapshots -m
    short-summary: Monthly snapshots count to keep
  - name: --hourly-minute
    short-summary: Which minute the hourly snapshot should be taken
  - name: --daily-minute
    short-summary: Which minute the daily snapshot should be taken
  - name: --daily-hour
    short-summary: Which hour in UTC timezone the daily snapshot should be taken
  - name: --weekly-minute
    short-summary: Which minute the weekly snapshot should be taken
  - name: --weekly-hour
    short-summary: Which hour in UTC timezone the weekly snapshot should be taken
  - name: --weekly-day
    short-summary: Which weekday the weekly snapshot should be taken, accepts a comma separated list of week day names in english
  - name: --monthly-minute
    short-summary: Which minute the monthly snapshot should be taken
  - name: --monthly-hour
    short-summary: Which hour in UTC timezone the monthly snapshot should be taken
  - name: --monthly-days
    short-summary: Which days of the month the weekly snapshot should be taken, accepts a comma separated list of days
  - name: --enabled -e
    short-summary: The property to decide policy is enabled or not
  - name: --tags
    short-summary: Space-separated tags in `key[=value]` format
examples:
  - name: Create an ANF snapshot policy
    text: >
        az netappfiles snapshot policy create -g mygroup --account-name myaccountname --snapshot-policy-name mysnapshotpolicyname -l westus2 --hourly-snapshots 1 --hourly-minute 5 --enabled true
"""

helps['netappfiles snapshot policy delete'] = """
type: command
short-summary: Delete the specified ANF snapshot policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --snapshot-policy-name
    short-summary: The name of the ANF snapshot policy
examples:
  - name: Delete an ANF snapshot policy
    text: >
        az netappfiles snapshot policy delete -g mygroup --account-name myaccname --snapshot-policy-name mysnapshotpolicyname
"""

helps['netappfiles snapshot policy list'] = """
type: command
short-summary: List the ANF snapshot policies for the specified account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
examples:
  - name: List the snapshot policy for the ANF account
    text: >
        az netappfiles snapshot policy list -g mygroup --account-name myname
"""

helps['netappfiles snapshot policy show'] = """
type: command
short-summary: Get the specified ANF snapshot policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --snapshot-policy-name
    short-summary: The name of the ANF snapshot policy
examples:
  - name: Get an ANF snapshot policy
    text: >
        az netappfiles snapshot policy show -g mygroup --account-name myaccname --snapshot-policy-name mysnapshotpolicyname
"""

helps['netappfiles snapshot policy volumes'] = """
type: command
short-summary: Get the all ANF volumes associated with snapshot policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --snapshot-policy-name
    short-summary: The name of the ANF snapshot policy
examples:
  - name: Get ANF volumes associated with snapshot policy
    text: >
        az netappfiles snapshot policy volumes -g mygroup --account-name myaccname --snapshot-policy-name mysnapshotpolicyname
"""

helps['netappfiles snapshot policy update'] = """
type: command
short-summary: Update the specified ANF snapshot policy.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --snapshot-policy-name
    short-summary: The name of the ANF snapshot policy
  - name: --hourly-snapshots -u
    short-summary: Hourly snapshots count to keep
  - name: --daily-snapshots -d
    short-summary: Daily snapshots count to keep
  - name: --weekly-snapshots -w
    short-summary: Weekly snapshots count to keep
  - name: --monthly-snapshots -m
    short-summary: Monthly snapshots count to keep
  - name: --hourly-minute
    short-summary: Which minute the hourly snapshot should be taken
  - name: --daily-minute
    short-summary: Which minute the daily snapshot should be taken
  - name: --daily-hour
    short-summary: Which hour in UTC timezone the daily snapshot should be taken
  - name: --weekly-minute
    short-summary: Which minute the weekly snapshot should be taken
  - name: --weekly-hour
    short-summary: Which hour in UTC timezone the weekly snapshot should be taken
  - name: --weekly-day
    short-summary: Which weekday the weekly snapshot should be taken, accepts a comma separated list of week day names in english
  - name: --monthly-minute
    short-summary: Which minute the monthly snapshot should be taken
  - name: --monthly-hour
    short-summary: Which hour in UTC timezone the monthly snapshot should be taken
  - name: --monthly-days
    short-summary: Which days of the month the weekly snapshot should be taken, accepts a comma separated list of days
  - name: --enabled -e
    short-summary: The property to decide policy is enabled or not
examples:
  - name: Update specific values for an ANF snapshot policy
    text: >
        az netappfiles snapshot policy update -g mygroup --account-name myaccountname --snapshot-policy-name mysnapshotpolicyname -l westus2 --daily-snapshots 1 --enabled false
"""

helps['netappfiles snapshot policy wait'] = """
type: command
short-summary: Wait for a snapshot policy operation.
"""


helps['netappfiles vault'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Vault Resources.
"""

helps['netappfiles vault list'] = """
type: command
short-summary: List the ANF vaults for NetApp Account.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
examples:
  - name: List the vaults of the ANF account
    text: >
        az netappfiles vault list -g mygroup --account-name myname
"""

helps['netappfiles subvolume'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Subvolume Resources.
"""

helps['netappfiles subvolume create'] = """
type: command
short-summary: Create a subvolume in the specified path or clones the subvolume specified in the parentPath
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --subvolume-name
    short-summary: The name of the ANF subvolume
  - name: --path
    short-summary: Path to the subvolume
  - name: --size
    short-summary: Size of the subvolume
  - name: --parent-path
    short-summary: Path to the parent subvolume
examples:
  - name: Create a ANF subvolume
    text: >
        az netappfiles subvolume create -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename --subvolume-name mysubvolumename
"""

helps['netappfiles subvolume update'] = """
type: command
short-summary: Update a specified ANF subvolume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --subvolume-name
    short-summary: The name of the ANF subvolume
  - name: --path
    short-summary: Path to the subvolume
  - name: --size
    short-summary: Size of the subvolume
examples:
  - name: Update a subvolume
    text: >
        az netappfiles subvolume update -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename --subvolume-name mysubvolumename
"""

helps['netappfiles subvolume list'] = """
type: command
short-summary: List all ANF subvolumes in the specified NetApp volume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
examples:
  - name: List all subvolumes of a ANF volume
    text: >
        az netappfiles subvolume list -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename
"""

helps['netappfiles subvolume show'] = """
type: command
short-summary: Get the path associated with a subvolumeName
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --subvolume-name
    short-summary: The name of the ANF subvolume
examples:
  - name: Get a subvolume of the ANF volume
    text: >
        az netappfiles subvolume show -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename --subvolume-name mysubvolumename
"""

helps['netappfiles subvolume delete'] = """
type: command
short-summary: Delete a specified ANF subvolume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --subvolume-name
    short-summary: The name of the ANF subvolume
examples:
  - name: Delete a subvolume of the ANF volume
    text: >
        az netappfiles subvolume delete -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename --subvolume-name mysubvolumename
"""

helps['netappfiles subvolume wait'] = """
type: command
short-summary: Wait for a subvolume operation.
"""

helps['netappfiles subvolume metadata'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Subvolume Metadata Resources.
"""

helps['netappfiles subvolume metadata show'] = """
type: command
short-summary: Get the specified ANF metadata of a subvolume.
parameters:
  - name: --account-name -a
    short-summary: The name of the ANF account
  - name: --pool-name -p
    short-summary: The name of the ANF pool
  - name: --volume-name -v
    short-summary: The name of the ANF volume
  - name: --subvolume-name
    short-summary: The name of the ANF subvolume
examples:
  - name: Get a metadata of an ANF subvolume
    text: >
        az netappfiles subvolume metadata show -g mygroup --account-name myaccountname  --pool-name mypoolname --volume-name myvolumename --subvolume-name mysubvolumename
"""


helps['netappfiles volume-group'] = """
type: group
short-summary: Manage Azure NetApp Files (ANF) Volume Group Resources.
"""

helps['netappfiles volume-group show'] = """
type: command
short-summary: Get the specified ANF Volume Group.
examples:
  - name: Get an ANF volume group
    text: >
        az netappfiles volume-group show -g mygroup --account-name myaccountname --volume-group-name myvolumegroupname
"""

helps['netappfiles volume-group list'] = """
type: command
short-summary: Get a list of ANF Volume Groups.
examples:
  - name: Get a list of ANF volume groups
    text: >
        az netappfiles volume-group list -g mygroup --account-name myaccountname
"""

helps['netappfiles volume-group create'] = """
type: command
short-summary: Create ANF Volume Groups.
parameters:
  - name: --vnet
    short-summary: The ARM Id or name of the vnet for the volumes
  - name: --ppg
    short-summary: The resource id of the Proximity Placement Group for volume placement
  - name: --sap-sid
    short-summary: The SAP system ID. Three characters long alpha-numeric string
  - name: --subnet
    short-summary: The delegated Subnet name
  - name: --location -l
    short-summary: ANF Location. If the resource group location is different than ANF location, ANF location needs to be specified
  - name: --tags
    short-summary: Space-separated tags in `key=value` format
  - name: --prefix
    short-summary: All volume names will be prefixed with the given text. The default values for prefix text depends on system role. For PRIMARY it will be `""` and HA it will be `"HA-"`.
  - name: --system-role
    short-summary: Type of role for the storage account. Primary indicates first of a SAP HANA Replication (HSR) setup or No HSR. High Availability (HA) specifies local scenario. Default is PRIMARY
  - name: --backup-nfsv3
    short-summary: Indicates if NFS Protocol version 3 is preferred for data backup and log backup volumes. Default is false
  - name: --data-repl-skd
    short-summary: Replication Schedule for data volume
  - name: --data-src-id
    short-summary: ResourceId of the data source volume
  - name: --shared-repl-skd
    short-summary: Replication Schedule for shared volume
  - name: --shared-src-id
    short-summary: ResourceId of the shared source volume
  - name: --data-backup-repl-skd
    short-summary: Replication Schedule for data backup volume
  - name: --data-backup-src-id
    short-summary: ResourceId of the data backup source volume
  - name: --log-backup-repl-skd
    short-summary: Replication Schedule for log backup volume
  - name: --log-backup-src-id
    short-summary: ResourceId of the log backup source volume
  - name: --kv-private-endpoint-id
    short-summary: The resource ID of private endpoint for KeyVault. It must reside in the same VNET as the volume. Only applicable if encryptionKeySource = 'Microsoft.KeyVault'.
examples:
  - name: Create ANF volume group
    text: >
        az netappfiles volume-group create -g mygroup --account-name myaccountname --pool-name mypoolname --volume-group-name myvolumegroupname --vnet myvnet --ppg myppg --sap-sid mysapsid
"""

helps['netappfiles volume-group wait'] = """
type: command
short-summary: Wait for a volume group to be created.
"""

helps['netappfiles resource'] = """
type: group
short-summary: Provides operations for Azure NetApp Files (ANF) Account Resources.
"""

helps['netappfiles resource query-region-info'] = """
type: command
short-summary: Provides storage to network proximity and logical zone mapping information.
parameters:
  - name: --location -l
    short-summary: The location
examples:
  - name: Describes region specific information.
    text: >
        az netappfiles resource query-region-info -l location
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=unused-argument, no-self-use, line-too-long, protected-access, too-few-public-methods
from knack.log import get_logger

from ._util import import_aaz_by_profile

logger = get_logger(__name__)


_VpnConnection = import_aaz_by_profile("network.vpn_connection")


class VpnConnectionUpdate(_VpnConnection.Update):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.ipsec_policies._registered = False

        return args_schema


class VpnConnectionDeviceConfigScriptShow(_VpnConnection.ShowDeviceConfigScript):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.device_family._required = True
        args_schema.firmware_version._required = True
        args_schema.vendor._required = True

        return args_schema


_VpnConnIpsecPolicy = import_aaz_by_profile("network.vpn_connection.ipsec_policy")


class VpnConnIpsecPolicyAdd(_VpnConnIpsecPolicy.Add):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.ipsec_policy_index._registered = False
        return args_schema


_VpnConnSharedKey = import_aaz_by_profile("network.vpn_connection.shared_key")


class VpnConnSharedKeyUpdate(_VpnConnSharedKey.Update):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.value._required = True
        return args_schema


def list_vpn_connections(cmd, resource_group_name, virtual_network_gateway_name=None):
    if virtual_network_gateway_name:
        return _VpnConnection.ListConnection(cli_ctx=cmd.cli_ctx)(
            command_args={"resource_group": resource_group_name,
                          "vnet_gateway": virtual_network_gateway_name})
    return _VpnConnection.List(cli_ctx=cmd.cli_ctx)(command_args={"resource_group": resource_group_name})


def clear_vpn_conn_ipsec_policies(cmd, resource_group_name, connection_name, no_wait=False):
    class VpnConnIpsecPoliciesClear(_VpnConnection.Update):

        def _output(self, *args, **kwargs):
            result = self.deserialize_output(self.ctx.vars.instance.properties.ipsec_policies, client_flatten=True)
            return result

        def pre_operations(self):
            args = self.ctx.args
            args.ipsec_policies = None
            args.use_policy_based_traffic_selectors = False

    ipsec_policies_args = {
        "resource_group": resource_group_name,
        "name": connection_name,
        "no_wait": no_wait,
    }
    return VpnConnIpsecPoliciesClear(cli_ctx=cmd.cli_ctx)(command_args=ipsec_policies_args)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.command_modules.lab.validators import validate_artifacts, validate_template_id
from azure.cli.core.util import get_json_object


def load_arguments(self, _):
    with self.argument_context('lab custom-image create') as c:
        c.argument('name', options_list=['--name', '-n'])

    with self.argument_context('lab vm create') as c:
        c.argument('name', options_list=['--name', '-n'])

        # Authentication related arguments
        for arg_name in ['admin_username', 'admin_password', 'authentication_type', 'ssh_key', 'generate_ssh_keys',
                         'saved_secret']:
            c.argument(arg_name, arg_group='Authentication')
        c.argument('generate_ssh_keys', action='store_true')

        # Add Artifacts from json object
        c.argument('artifacts', type=get_json_object)

        # Image related arguments
        c.ignore('os_type', 'gallery_image_reference', 'custom_image_id')

        # Network related arguments
        for arg_name in ['ip_configuration', 'subnet', 'vnet_name']:
            c.argument(arg_name, arg_group='Network')

        c.ignore('lab_subnet_name', 'lab_virtual_network_id', 'disallow_public_ip_address', 'network_interface')

        # Creating VM in the different location then lab is an officially unsupported scenario
        c.ignore('location')
        c.argument('allow_claim', action='store_true')

    with self.argument_context('lab vm list') as c:
        for arg_name in ['filters', 'all', 'claimable', 'environment']:
            c.argument(arg_name, arg_group='Filter')

        for arg_name in ['all', 'claimable']:
            c.argument(arg_name, action='store_true')

    with self.argument_context('lab vm claim') as c:
        c.argument('name', options_list=['--name', '-n'], id_part='child_name_1')
        c.argument('lab_name', id_part='name')

    with self.argument_context('lab vm apply-artifacts') as c:
        c.argument('artifacts', type=get_json_object, validator=validate_artifacts)
        c.argument('name', options_list=['--name', '-n'])

    with self.argument_context('lab formula') as c:
        c.argument('name', options_list=['--name', '-n'])

    with self.argument_context('lab secret') as c:
        from azure.mgmt.devtestlabs.models import Secret

        c.argument('name', options_list=['--name', '-n'])
        c.argument('secret', options_list=['--value'], type=lambda x: Secret(value=x))
        c.ignore('user_name')

    with self.argument_context('lab formula export-artifacts') as c:
        # Exporting artifacts does not need expand filter
        c.ignore('expand')

    with self.argument_context('lab environment') as c:
        c.argument('name', options_list=['--name', '-n'])
        c.ignore('user_name')

    with self.argument_context('lab environment create') as c:
        c.argument('arm_template', validator=validate_template_id)
        c.argument('parameters', type=get_json_object)

    with self.argument_context('lab arm-template') as c:
        c.argument('name', options_list=['--name', '-n'])

    with self.argument_context('lab arm-template show') as c:
        c.argument('export_parameters', action='store_true')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network watcher connection-monitor test-configuration add",
    is_preview=True,
)
class Add(AAZCommand):
    """Add a test configuration to a connection monitor.

    :example: Add a test configuration with HTTP supported
        az network watcher connection-monitor test-configuration add --connection-monitor MyConnectionMonitor --location westus --name MyHTTPTestConfiguration --test-groups DefaultTestGroup --protocol Http --http-request-header name=Host value=bing.com --http- request-header name=UserAgent value=Edge

    :example: Add a test configuration with TCP supported
        az network watcher connection-monitor test-configuration add --connection-monitor MyConnectionMonitor --location westus --name MyHTTPTestConfiguration --test-groups TCPTestGroup DefaultTestGroup --protocol Tcp --tcp-port 4096
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/connectionmonitors/{}", "2022-01-01", "properties.testConfigurations[]"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        self.SubresourceSelector(ctx=self.ctx, name="subresource")
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.connection_monitor = AAZStrArg(
            options=["--connection-monitor"],
            help="Connection monitor name.",
            required=True,
        )
        _args_schema.watcher_name = AAZStrArg(
            options=["--watcher-name"],
            help="The name of the Network Watcher resource.",
            required=True,
        )
        _args_schema.watcher_rg = AAZResourceGroupNameArg(
            options=["-g", "--watcher-rg"],
            help="Name of resource group. You can configure the default group using `az configure --defaults group=<name>`.",
            required=True,
        )
        _args_schema.test_configuration_name = AAZStrArg(
            options=["-n", "--name", "--test-configuration-name"],
            help="The name of the connection monitor test configuration.",
            required=True,
        )
        _args_schema.preferred_ip_version = AAZStrArg(
            options=["--preferred-ip-version"],
            help="The preferred IP version to use in test evaluation. The connection monitor may choose to use a different version depending on other parameters.  Allowed values: IPv4, IPv6.",
            enum={"IPv4": "IPv4", "IPv6": "IPv6"},
        )
        _args_schema.protocol = AAZStrArg(
            options=["--protocol"],
            help="The protocol to use in test evaluation.  Allowed values: Http, Icmp, Tcp.",
            required=True,
            enum={"Http": "Http", "Icmp": "Icmp", "Tcp": "Tcp"},
        )
        _args_schema.threshold_failed_percent = AAZIntArg(
            options=["--threshold-failed-percent"],
            help="The maximum percentage of failed checks permitted for a test to evaluate as successful.",
        )
        _args_schema.threshold_round_trip_time = AAZFloatArg(
            options=["--threshold-round-trip-time"],
            help="The maximum round-trip time in milliseconds permitted for a test to evaluate as successful.",
        )
        _args_schema.frequency = AAZIntArg(
            options=["--frequency"],
            help="The frequency of test evaluation, in seconds.  Default: 60.",
        )

        # define Arg Group "HTTP Protocol"

        _args_schema = cls._args_schema
        _args_schema.http_method = AAZStrArg(
            options=["--http-method"],
            arg_group="HTTP Protocol",
            help="The HTTP method to use. Allowed values: Get, Post.",
            enum={"Get": "Get", "Post": "Post"},
        )
        _args_schema.http_path = AAZStrArg(
            options=["--http-path"],
            arg_group="HTTP Protocol",
            help="The path component of the URI. For instance, \"/dir1/dir2\".",
        )
        _args_schema.http_port = AAZIntArg(
            options=["--http-port"],
            arg_group="HTTP Protocol",
            help="The port to connect to.",
            fmt=AAZIntArgFormat(
                maximum=65535,
                minimum=0,
            ),
        )
        _args_schema.http_prefer_https = AAZBoolArg(
            options=["--http-prefer-https"],
            arg_group="HTTP Protocol",
            help="Value indicating whether HTTPS is preferred over HTTP in cases where the choice is not explicit.",
        )
        _args_schema.http_request_headers = AAZListArg(
            options=["--http-request-headers"],
            arg_group="HTTP Protocol",
            help="The HTTP headers to transmit with the request.",
        )
        _args_schema.http_valid_status_codes = AAZListArg(
            options=["--http-valid-status-codes"],
            arg_group="HTTP Protocol",
            help="HTTP status codes to consider successful. For instance, \"2xx,301-304,418\".",
        )

        http_request_headers = cls._args_schema.http_request_headers
        http_request_headers.Element = AAZObjectArg()

        _element = cls._args_schema.http_request_headers.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name in HTTP header.",
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="The value in HTTP header.",
        )

        http_valid_status_codes = cls._args_schema.http_valid_status_codes
        http_valid_status_codes.Element = AAZStrArg()

        # define Arg Group "ICMP Protocol"

        _args_schema = cls._args_schema
        _args_schema.icmp_disable_trace_route = AAZBoolArg(
            options=["--icmp-disable-trace-route"],
            arg_group="ICMP Protocol",
            help="Value indicating whether path evaluation with trace route should be disabled. false is default.  Allowed values: false, true.",
        )

        # define Arg Group "TCP Protocol"

        _args_schema = cls._args_schema
        _args_schema.tcp_port_behavior = AAZStrArg(
            options=["--tcp-port-behavior"],
            arg_group="TCP Protocol",
            help="Destination port behavior.  Allowed values: ListenIfAvailable,  None.",
            enum={"ListenIfAvailable": "ListenIfAvailable", "None": "None"},
        )
        _args_schema.tcp_disable_trace_route = AAZBoolArg(
            options=["--tcp-disable-trace-route"],
            arg_group="TCP Protocol",
            help="Value indicating whether path evaluation with trace route should be disabled. false is default.  Allowed values: false, true.",
        )
        _args_schema.tcp_port = AAZIntArg(
            options=["--tcp-port"],
            arg_group="TCP Protocol",
            help="The port to connect to.",
            fmt=AAZIntArgFormat(
                maximum=65535,
                minimum=0,
            ),
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ConnectionMonitorsGet(ctx=self.ctx)()
        self.pre_instance_create()
        self.InstanceCreateByJson(ctx=self.ctx)()
        self.post_instance_create(self.ctx.selectors.subresource.required())
        yield self.ConnectionMonitorsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_create(self):
        pass

    @register_callback
    def post_instance_create(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.selectors.subresource.required(), client_flatten=True)
        return result

    class SubresourceSelector(AAZJsonSelector):

        def _get(self):
            result = self.ctx.vars.instance
            result = result.properties.testConfigurations
            filters = enumerate(result)
            filters = filter(
                lambda e: e[1].name == self.ctx.args.test_configuration_name,
                filters
            )
            idx = next(filters)[0]
            return result[idx]

        def _set(self, value):
            result = self.ctx.vars.instance
            result = result.properties.testConfigurations
            filters = enumerate(result)
            filters = filter(
                lambda e: e[1].name == self.ctx.args.test_configuration_name,
                filters
            )
            idx = next(filters, [len(result)])[0]
            result[idx] = value
            return

    class ConnectionMonitorsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/connectionMonitors/{connectionMonitorName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "connectionMonitorName", self.ctx.args.connection_monitor,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.watcher_rg,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _AddHelper._build_schema_connection_monitor_result_read(cls._schema_on_200)

            return cls._schema_on_200

    class ConnectionMonitorsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/connectionMonitors/{connectionMonitorName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "connectionMonitorName", self.ctx.args.connection_monitor,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.watcher_rg,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _AddHelper._build_schema_connection_monitor_result_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceCreateByJson(AAZJsonInstanceCreateOperation):

        def __call__(self, *args, **kwargs):
            self.ctx.selectors.subresource.set(self._create_instance())

        def _create_instance(self):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType
            )
            _builder.set_prop("httpConfiguration", AAZObjectType)
            _builder.set_prop("icmpConfiguration", AAZObjectType)
            _builder.set_prop("name", AAZStrType, ".test_configuration_name", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("preferredIPVersion", AAZStrType, ".preferred_ip_version")
            _builder.set_prop("protocol", AAZStrType, ".protocol", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("successThreshold", AAZObjectType)
            _builder.set_prop("tcpConfiguration", AAZObjectType)
            _builder.set_prop("testFrequencySec", AAZIntType, ".frequency")

            http_configuration = _builder.get(".httpConfiguration")
            if http_configuration is not None:
                http_configuration.set_prop("method", AAZStrType, ".http_method")
                http_configuration.set_prop("path", AAZStrType, ".http_path")
                http_configuration.set_prop("port", AAZIntType, ".http_port")
                http_configuration.set_prop("preferHTTPS", AAZBoolType, ".http_prefer_https")
                http_configuration.set_prop("requestHeaders", AAZListType, ".http_request_headers")
                http_configuration.set_prop("validStatusCodeRanges", AAZListType, ".http_valid_status_codes")

            request_headers = _builder.get(".httpConfiguration.requestHeaders")
            if request_headers is not None:
                request_headers.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".httpConfiguration.requestHeaders[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("value", AAZStrType, ".value")

            valid_status_code_ranges = _builder.get(".httpConfiguration.validStatusCodeRanges")
            if valid_status_code_ranges is not None:
                valid_status_code_ranges.set_elements(AAZStrType, ".")

            icmp_configuration = _builder.get(".icmpConfiguration")
            if icmp_configuration is not None:
                icmp_configuration.set_prop("disableTraceRoute", AAZBoolType, ".icmp_disable_trace_route")

            success_threshold = _builder.get(".successThreshold")
            if success_threshold is not None:
                success_threshold.set_prop("checksFailedPercent", AAZIntType, ".threshold_failed_percent")
                success_threshold.set_prop("roundTripTimeMs", AAZFloatType, ".threshold_round_trip_time")

            tcp_configuration = _builder.get(".tcpConfiguration")
            if tcp_configuration is not None:
                tcp_configuration.set_prop("destinationPortBehavior", AAZStrType, ".tcp_port_behavior")
                tcp_configuration.set_prop("disableTraceRoute", AAZBoolType, ".tcp_disable_trace_route")
                tcp_configuration.set_prop("port", AAZIntType, ".tcp_port")

            return _instance_value


class _AddHelper:
    """Helper class for Add"""

    _schema_connection_monitor_endpoint_scope_item_read = None

    @classmethod
    def _build_schema_connection_monitor_endpoint_scope_item_read(cls, _schema):
        if cls._schema_connection_monitor_endpoint_scope_item_read is not None:
            _schema.address = cls._schema_connection_monitor_endpoint_scope_item_read.address
            return

        cls._schema_connection_monitor_endpoint_scope_item_read = _schema_connection_monitor_endpoint_scope_item_read = AAZObjectType()

        connection_monitor_endpoint_scope_item_read = _schema_connection_monitor_endpoint_scope_item_read
        connection_monitor_endpoint_scope_item_read.address = AAZStrType()

        _schema.address = cls._schema_connection_monitor_endpoint_scope_item_read.address

    _schema_connection_monitor_result_read = None

    @classmethod
    def _build_schema_connection_monitor_result_read(cls, _schema):
        if cls._schema_connection_monitor_result_read is not None:
            _schema.etag = cls._schema_connection_monitor_result_read.etag
            _schema.id = cls._schema_connection_monitor_result_read.id
            _schema.location = cls._schema_connection_monitor_result_read.location
            _schema.name = cls._schema_connection_monitor_result_read.name
            _schema.properties = cls._schema_connection_monitor_result_read.properties
            _schema.tags = cls._schema_connection_monitor_result_read.tags
            _schema.type = cls._schema_connection_monitor_result_read.type
            return

        cls._schema_connection_monitor_result_read = _schema_connection_monitor_result_read = AAZObjectType()

        connection_monitor_result_read = _schema_connection_monitor_result_read
        connection_monitor_result_read.etag = AAZStrType(
            flags={"read_only": True},
        )
        connection_monitor_result_read.id = AAZStrType(
            flags={"read_only": True},
        )
        connection_monitor_result_read.location = AAZStrType()
        connection_monitor_result_read.name = AAZStrType(
            flags={"read_only": True},
        )
        connection_monitor_result_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        connection_monitor_result_read.tags = AAZDictType()
        connection_monitor_result_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_connection_monitor_result_read.properties
        properties.auto_start = AAZBoolType(
            serialized_name="autoStart",
        )
        properties.connection_monitor_type = AAZStrType(
            serialized_name="connectionMonitorType",
            flags={"read_only": True},
        )
        properties.destination = AAZObjectType()
        properties.endpoints = AAZListType()
        properties.monitoring_interval_in_seconds = AAZIntType(
            serialized_name="monitoringIntervalInSeconds",
        )
        properties.monitoring_status = AAZStrType(
            serialized_name="monitoringStatus",
            flags={"read_only": True},
        )
        properties.notes = AAZStrType()
        properties.outputs = AAZListType()
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.source = AAZObjectType()
        properties.start_time = AAZStrType(
            serialized_name="startTime",
            flags={"read_only": True},
        )
        properties.test_configurations = AAZListType(
            serialized_name="testConfigurations",
        )
        properties.test_groups = AAZListType(
            serialized_name="testGroups",
        )

        destination = _schema_connection_monitor_result_read.properties.destination
        destination.address = AAZStrType()
        destination.port = AAZIntType()
        destination.resource_id = AAZStrType(
            serialized_name="resourceId",
        )

        endpoints = _schema_connection_monitor_result_read.properties.endpoints
        endpoints.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.endpoints.Element
        _element.address = AAZStrType()
        _element.coverage_level = AAZStrType(
            serialized_name="coverageLevel",
        )
        _element.filter = AAZObjectType()
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.resource_id = AAZStrType(
            serialized_name="resourceId",
        )
        _element.scope = AAZObjectType()
        _element.type = AAZStrType()

        filter = _schema_connection_monitor_result_read.properties.endpoints.Element.filter
        filter.items = AAZListType()
        filter.type = AAZStrType()

        items = _schema_connection_monitor_result_read.properties.endpoints.Element.filter.items
        items.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.endpoints.Element.filter.items.Element
        _element.address = AAZStrType()
        _element.type = AAZStrType()

        scope = _schema_connection_monitor_result_read.properties.endpoints.Element.scope
        scope.exclude = AAZListType()
        scope.include = AAZListType()

        exclude = _schema_connection_monitor_result_read.properties.endpoints.Element.scope.exclude
        exclude.Element = AAZObjectType()
        cls._build_schema_connection_monitor_endpoint_scope_item_read(exclude.Element)

        include = _schema_connection_monitor_result_read.properties.endpoints.Element.scope.include
        include.Element = AAZObjectType()
        cls._build_schema_connection_monitor_endpoint_scope_item_read(include.Element)

        outputs = _schema_connection_monitor_result_read.properties.outputs
        outputs.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.outputs.Element
        _element.type = AAZStrType()
        _element.workspace_settings = AAZObjectType(
            serialized_name="workspaceSettings",
        )

        workspace_settings = _schema_connection_monitor_result_read.properties.outputs.Element.workspace_settings
        workspace_settings.workspace_resource_id = AAZStrType(
            serialized_name="workspaceResourceId",
        )

        source = _schema_connection_monitor_result_read.properties.source
        source.port = AAZIntType()
        source.resource_id = AAZStrType(
            serialized_name="resourceId",
            flags={"required": True},
        )

        test_configurations = _schema_connection_monitor_result_read.properties.test_configurations
        test_configurations.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.test_configurations.Element
        _element.http_configuration = AAZObjectType(
            serialized_name="httpConfiguration",
        )
        _element.icmp_configuration = AAZObjectType(
            serialized_name="icmpConfiguration",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.preferred_ip_version = AAZStrType(
            serialized_name="preferredIPVersion",
        )
        _element.protocol = AAZStrType(
            flags={"required": True},
        )
        _element.success_threshold = AAZObjectType(
            serialized_name="successThreshold",
        )
        _element.tcp_configuration = AAZObjectType(
            serialized_name="tcpConfiguration",
        )
        _element.test_frequency_sec = AAZIntType(
            serialized_name="testFrequencySec",
        )

        http_configuration = _schema_connection_monitor_result_read.properties.test_configurations.Element.http_configuration
        http_configuration.method = AAZStrType()
        http_configuration.path = AAZStrType()
        http_configuration.port = AAZIntType()
        http_configuration.prefer_https = AAZBoolType(
            serialized_name="preferHTTPS",
        )
        http_configuration.request_headers = AAZListType(
            serialized_name="requestHeaders",
        )
        http_configuration.valid_status_code_ranges = AAZListType(
            serialized_name="validStatusCodeRanges",
        )

        request_headers = _schema_connection_monitor_result_read.properties.test_configurations.Element.http_configuration.request_headers
        request_headers.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.test_configurations.Element.http_configuration.request_headers.Element
        _element.name = AAZStrType()
        _element.value = AAZStrType()

        valid_status_code_ranges = _schema_connection_monitor_result_read.properties.test_configurations.Element.http_configuration.valid_status_code_ranges
        valid_status_code_ranges.Element = AAZStrType()

        icmp_configuration = _schema_connection_monitor_result_read.properties.test_configurations.Element.icmp_configuration
        icmp_configuration.disable_trace_route = AAZBoolType(
            serialized_name="disableTraceRoute",
        )

        success_threshold = _schema_connection_monitor_result_read.properties.test_configurations.Element.success_threshold
        success_threshold.checks_failed_percent = AAZIntType(
            serialized_name="checksFailedPercent",
        )
        success_threshold.round_trip_time_ms = AAZFloatType(
            serialized_name="roundTripTimeMs",
        )

        tcp_configuration = _schema_connection_monitor_result_read.properties.test_configurations.Element.tcp_configuration
        tcp_configuration.destination_port_behavior = AAZStrType(
            serialized_name="destinationPortBehavior",
        )
        tcp_configuration.disable_trace_route = AAZBoolType(
            serialized_name="disableTraceRoute",
        )
        tcp_configuration.port = AAZIntType()

        test_groups = _schema_connection_monitor_result_read.properties.test_groups
        test_groups.Element = AAZObjectType()

        _element = _schema_connection_monitor_result_read.properties.test_groups.Element
        _element.destinations = AAZListType(
            flags={"required": True},
        )
        _element.disable = AAZBoolType()
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.sources = AAZListType(
            flags={"required": True},
        )
        _element.test_configurations = AAZListType(
            serialized_name="testConfigurations",
            flags={"required": True},
        )

        destinations = _schema_connection_monitor_result_read.properties.test_groups.Element.destinations
        destinations.Element = AAZStrType()

        sources = _schema_connection_monitor_result_read.properties.test_groups.Element.sources
        sources.Element = AAZStrType()

        test_configurations = _schema_connection_monitor_result_read.properties.test_groups.Element.test_configurations
        test_configurations.Element = AAZStrType()

        tags = _schema_connection_monitor_result_read.tags
        tags.Element = AAZStrType()

        _schema.etag = cls._schema_connection_monitor_result_read.etag
        _schema.id = cls._schema_connection_monitor_result_read.id
        _schema.location = cls._schema_connection_monitor_result_read.location
        _schema.name = cls._schema_connection_monitor_result_read.name
        _schema.properties = cls._schema_connection_monitor_result_read.properties
        _schema.tags = cls._schema_connection_monitor_result_read.tags
        _schema.type = cls._schema_connection_monitor_result_read.type


__all__ = ["Add"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=no-self-use, line-too-long, protected-access, too-few-public-methods, unused-argument
from azure.cli.core.aaz import AAZResourceIdArgFormat, has_value, AAZListArg, AAZResourceIdArg, \
    AAZStrArg, AAZListArgFormat
from azure.cli.core.aaz.utils import assign_aaz_list_arg
from knack.log import get_logger

from ._util import import_aaz_by_profile

logger = get_logger(__name__)


_NIC = import_aaz_by_profile("network.nic")


class NICCreate(_NIC.Create):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet_name = AAZStrArg(
            options=["--vnet-name"],
            arg_group="IP Configuration",
            help="Name of the virtual network.",
        )
        args_schema.subnet = AAZResourceIdArg(
            options=["--subnet"],
            arg_group="IP Configuration",
            help="Name or ID of an existing subnet. If name specified, please also specify `--vnet-name`; "
                 "If you want to use an existing subnet in other resource group, "
                 "please provide the ID instead of the name of the subnet.",
            required=True,
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/virtualNetworks/{vnet_name}/subnets/{}",
            ),
        )
        args_schema.application_security_groups = AAZListArg(
            options=["--application-security-groups", "--asgs"],
            arg_group="IP Configuration",
            help="Space-separated list of application security groups.",
        )
        args_schema.application_security_groups.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationSecurityGroups/{}",
            ),
        )
        args_schema.private_ip_address = AAZStrArg(
            options=["--private-ip-address"],
            arg_group="IP Configuration",
            help="Static private IP address to use.",
        )
        args_schema.private_ip_address_version = AAZStrArg(
            options=["--private-ip-address-version"],
            arg_group="IP Configuration",
            help="Version of private IP address to use.",
            enum=["IPv4", "IPv6"],
            default="IPv4",
        )
        args_schema.public_ip_address = AAZResourceIdArg(
            options=["--public-ip-address"],
            arg_group="IP Configuration",
            help="Name or ID of an existing public IP address.",
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/"
                         "publicIPAddresses/{}"
            ),
        )
        args_schema.gateway_name = AAZStrArg(
            options=["--gateway-name"],
            arg_group="Application Gateway",
            help="Name of the application gateway."
        )
        args_schema.app_gateway_address_pools = AAZListArg(
            options=["--app-gateway-address-pools", "--ag-address-pools"],
            arg_group="Application Gateway",
            help="Space-separated list of names or IDs of application gateway backend address pools to "
                 "associate with the NIC. If names are used, `--gateway-name` must be specified.",
        )
        args_schema.app_gateway_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationGateways/{gateway_name}/backendAddressPools/{}",
            ),
        )
        args_schema.lb_name = AAZStrArg(
            options=["--lb-name"],
            arg_group="Load Balancer",
            help="Name of the load balancer",
        )
        args_schema.lb_address_pools = AAZListArg(
            options=["--lb-address-pools"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer address pools to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
        )
        args_schema.lb_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/backendAddressPools/{}",
            ),
        )
        args_schema.lb_inbound_nat_rules = AAZListArg(
            options=["--lb-inbound-nat-rules"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer inbound NAT rules to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
        )
        args_schema.lb_inbound_nat_rules.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/inboundNatRules/{}",
            ),
        )
        args_schema.network_security_group = AAZResourceIdArg(
            options=["--network-security-group"],
            help="Name or ID of an existing network security group",
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/networkSecurityGroups/{}",
            ),
        )
        args_schema.ip_configurations._registered = False
        args_schema.nsg._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.network_security_group):
            args.nsg.id = args.network_security_group
        ip_configuration = {
            "name": "ipconfig1",
            "private_ip_address": args.private_ip_address,
            "private_ip_address_version": args.private_ip_address_version,  # when address doesn't exist, version should be ipv4 (default)
            "private_ip_allocation_method": "Static" if has_value(args.private_ip_address) else "Dynamic",
            "subnet": {"id": args.subnet} if has_value(args.subnet) else None,
            "public_ip_address": {"id": args.public_ip_address} if has_value(args.public_ip_address) else None,
            "application_security_groups": [{"id": x} for x in args.application_security_groups] if has_value(args.application_security_groups) else None,
            "application_gateway_backend_address_pools": [{"id": x} for x in args.app_gateway_address_pools] if has_value(args.app_gateway_address_pools) else None,
            "load_balancer_backend_address_pools": [{"id": x} for x in args.lb_address_pools] if has_value(args.lb_address_pools) else None,
            "load_balancer_inbound_nat_rules": [{"id": x} for x in args.lb_inbound_nat_rules] if has_value(args.lb_inbound_nat_rules) else None,
        }
        args.ip_configurations = [ip_configuration]

    def _output(self, *args, **kwargs):
        result = super()._output(*args, **kwargs)
        return {"NewNIC": result}


class NICUpdate(_NIC.Update):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        class EmptyListArgFormat(AAZListArgFormat):
            def __call__(self, ctx, value):
                if value.to_serialized_data() == [""]:
                    logger.warning("It's recommended to detach it by null, empty string (\"\") will be deprecated.")
                    value._data = None
                return super().__call__(ctx, value)

        class EmptyResourceIdArgFormat(AAZResourceIdArgFormat):
            def __call__(self, ctx, value):
                if value._data == "":
                    logger.warning("It's recommended to detach it by null, empty string (\"\") will be deprecated.")
                    value._data = None
                return super().__call__(ctx, value)

        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.network_security_group = AAZResourceIdArg(
            options=["--network-security-group"],
            help="Name or ID of an existing network security group",
            fmt=EmptyResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/networkSecurityGroups/{}",
            ),
            nullable=True,
        )
        args_schema.dns_servers._fmt = EmptyListArgFormat()
        args_schema.nsg._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.network_security_group):
            args.nsg.id = args.network_security_group
        if has_value(args.internal_dns_name) and args.internal_dns_name == "":
            args.internal_dns_name = None

    def post_instance_update(self, instance):
        if not has_value(instance.properties.network_security_group.id):
            instance.properties.network_security_group = None


_NICIPConfig = import_aaz_by_profile("network.nic.ip_config")


class NICIPConfigCreate(_NICIPConfig.Create):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet_name = AAZStrArg(
            options=["--vnet-name"],
            arg_group="IP Configuration",
            help="Name of the virtual network.",
        )
        args_schema.subnet._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                     "/virtualNetworks/{vnet_name}/subnets/{}",
        )
        args_schema.public_ip_address._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/"
                     "publicIPAddresses/{}"
        )
        args_schema.application_security_groups = AAZListArg(
            options=["--application-security-groups", "--asgs"],
            arg_group="IP Configuration",
            help="Space-separated list of application security groups.",
        )
        args_schema.application_security_groups.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationSecurityGroups/{}",
            ),
        )
        args_schema.gateway_name = AAZStrArg(
            options=["--gateway-name"],
            arg_group="Application Gateway",
            help="Name of the application gateway."
        )
        args_schema.app_gateway_address_pools = AAZListArg(
            options=["--app-gateway-address-pools", "--ag-address-pools"],
            arg_group="Application Gateway",
            help="Space-separated list of names or IDs of application gateway backend address pools to "
                 "associate with the NIC. If names are used, `--gateway-name` must be specified.",
        )
        args_schema.app_gateway_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationGateways/{gateway_name}/backendAddressPools/{}",
            ),
        )
        args_schema.lb_name = AAZStrArg(
            options=["--lb-name"],
            arg_group="Load Balancer",
            help="Name of the load balancer",
        )
        args_schema.lb_address_pools = AAZListArg(
            options=["--lb-address-pools"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer address pools to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
        )
        args_schema.lb_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/backendAddressPools/{}",
            ),
        )
        args_schema.lb_inbound_nat_rules = AAZListArg(
            options=["--lb-inbound-nat-rules"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer inbound NAT rules to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
        )
        args_schema.lb_inbound_nat_rules.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/inboundNatRules/{}",
            ),
        )
        args_schema.application_gateway_backend_address_pools._registered = False
        args_schema.load_balancer_backend_address_pools._registered = False
        args_schema.load_balancer_inbound_nat_rules._registered = False
        args_schema.private_ip_allocation_method._registered = False
        args_schema.asgs_obj._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.private_ip_allocation_method = "Static" if has_value(args.private_ip_address) else "Dynamic"

        args.asgs_obj = assign_aaz_list_arg(
            args.asgs_obj,
            args.application_security_groups,
            element_transformer=lambda _, asg_id: {"id": asg_id}
        )
        args.application_gateway_backend_address_pools = assign_aaz_list_arg(
            args.application_gateway_backend_address_pools,
            args.app_gateway_address_pools,
            element_transformer=lambda _, pool_id: {"id": pool_id}
        )
        args.load_balancer_backend_address_pools = assign_aaz_list_arg(
            args.load_balancer_backend_address_pools,
            args.lb_address_pools,
            element_transformer=lambda _, pool_id: {"id": pool_id}
        )
        args.load_balancer_inbound_nat_rules = assign_aaz_list_arg(
            args.load_balancer_inbound_nat_rules,
            args.lb_inbound_nat_rules,
            element_transformer=lambda _, rule_id: {"id": rule_id}
        )

    def pre_instance_create(self):
        args = self.ctx.args
        instance = self.ctx.vars.instance
        if args.private_ip_address_version.to_serialized_data().lower() == "ipv4" and not has_value(args.subnet):
            primary = next(x for x in instance.properties.ip_configurations if x.properties.primary)
            args.subnet = primary.properties.subnet.id
        if args.make_primary.to_serialized_data():
            for config in instance.properties.ip_configurations:
                config.properties.primary = False


class NICIPConfigUpdate(_NICIPConfig.Update):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):

        class EmptyListArgFormat(AAZListArgFormat):
            def __call__(self, ctx, value):
                if value.to_serialized_data() == [""]:
                    logger.warning("It's recommended to detach it by null, empty string (\"\") will be deprecated.")
                    value._data = None
                return super().__call__(ctx, value)

        class EmptyResourceIdArgFormat(AAZResourceIdArgFormat):
            def __call__(self, ctx, value):
                if value._data == "":
                    logger.warning("It's recommended to detach it by null, empty string (\"\") will be deprecated.")
                    value._data = None
                return super().__call__(ctx, value)

        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.vnet_name = AAZStrArg(
            options=["--vnet-name"],
            arg_group="IP Configuration",
            help="Name of the virtual network.",
            nullable=True,
        )
        args_schema.subnet._fmt = EmptyResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                     "/virtualNetworks/{vnet_name}/subnets/{}",
        )
        args_schema.public_ip_address._fmt = EmptyResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network/"
                     "publicIPAddresses/{}"
        )
        args_schema.application_security_groups = AAZListArg(
            options=["--application-security-groups", "--asgs"],
            arg_group="IP Configuration",
            help="Space-separated list of application security groups.",
            fmt=EmptyListArgFormat(),
            nullable=True,
        )
        args_schema.application_security_groups.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationSecurityGroups/{}",
            ),
            nullable=True,
        )
        args_schema.gateway_name = AAZStrArg(
            options=["--gateway-name"],
            arg_group="Application Gateway",
            help="Name of the application gateway.",
            nullable=True,
        )
        args_schema.app_gateway_address_pools = AAZListArg(
            options=["--app-gateway-address-pools", "--ag-address-pools"],
            arg_group="Application Gateway",
            help="Space-separated list of names or IDs of application gateway backend address pools to "
                 "associate with the NIC. If names are used, `--gateway-name` must be specified.",
            fmt=EmptyListArgFormat(),
            nullable=True,
        )
        args_schema.app_gateway_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/applicationGateways/{gateway_name}/backendAddressPools/{}",
            ),
            nullable=True,
        )
        args_schema.lb_name = AAZStrArg(
            options=["--lb-name"],
            arg_group="Load Balancer",
            help="Name of the load balancer",
            nullable=True,
        )
        args_schema.lb_address_pools = AAZListArg(
            options=["--lb-address-pools"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer address pools to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
            fmt=EmptyListArgFormat(),
            nullable=True,
        )
        args_schema.lb_address_pools.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/backendAddressPools/{}",
            ),
            nullable=True,
        )
        args_schema.lb_inbound_nat_rules = AAZListArg(
            options=["--lb-inbound-nat-rules"],
            arg_group="Load Balancer",
            help="Space-separated list of names or IDs of load balancer inbound NAT rules to associate with the NIC. "
                 "If names are used, `--lb-name` must be specified.",
            fmt=EmptyListArgFormat(),
            nullable=True,
        )
        args_schema.lb_inbound_nat_rules.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                         "/loadBalancers/{lb_name}/inboundNatRules/{}",
            ),
            nullable=True,
        )
        args_schema.application_gateway_backend_address_pools._registered = False
        args_schema.load_balancer_backend_address_pools._registered = False
        args_schema.load_balancer_inbound_nat_rules._registered = False
        args_schema.private_ip_allocation_method._registered = False
        args_schema.asgs_obj._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if has_value(args.private_ip_address):
            if args.private_ip_address is None or args.private_ip_address == "":
                # switch private IP address allocation to dynamic if empty string is used
                args.private_ip_address = None
                args.private_ip_allocation_method = "Dynamic"
                args.private_ip_address_version = "IPv4"
            else:
                # if specific address provided, allocation is static
                args.private_ip_allocation_method = "Static"

    def pre_instance_update(self, instance):
        args = self.ctx.args
        instance = self.ctx.vars.instance
        args.asgs_obj = assign_aaz_list_arg(
            args.asgs_obj,
            args.application_security_groups,
            element_transformer=lambda _, asg_id: {"id": asg_id}
        )
        args.application_gateway_backend_address_pools = assign_aaz_list_arg(
            args.application_gateway_backend_address_pools,
            args.app_gateway_address_pools,
            element_transformer=lambda _, pool_id: {"id": pool_id}
        )
        args.load_balancer_backend_address_pools = assign_aaz_list_arg(
            args.load_balancer_backend_address_pools,
            args.lb_address_pools,
            element_transformer=lambda _, pool_id: {"id": pool_id}
        )
        args.load_balancer_inbound_nat_rules = assign_aaz_list_arg(
            args.load_balancer_inbound_nat_rules,
            args.lb_inbound_nat_rules,
            element_transformer=lambda _, rule_id: {"id": rule_id}
        )
        # all ip configurations must belong to the same asgs
        is_primary = args.make_primary.to_serialized_data()
        for config in instance.properties.ip_configurations:
            if is_primary:
                config.properties.primary = False
            config.properties.application_security_groups = args.asgs_obj

    def post_instance_update(self, instance):
        if not has_value(instance.properties.subnet.id):
            instance.properties.subnet = None
        if not has_value(instance.properties.public_ip_address.id):
            instance.properties.public_ip_address = None


_LBPool = import_aaz_by_profile("network.nic.ip_config.lb_pool")

_AGPool = import_aaz_by_profile("network.nic.ip_config.ag_pool")


def add_nic_ip_config_address_pool(cmd, resource_group_name, network_interface_name, ip_config_name,
                                   backend_address_pool, load_balancer_name=None, application_gateway_name=None):

    class LBPoolAdd(_LBPool.Add):
        def _output(self, *args, **kwargs):
            result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
            return result["ipConfigurations"][0]

    class AGPoolAdd(_AGPool.Add):
        def _output(self, *args, **kwargs):
            result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
            return result["ipConfigurations"][0]

    arguments = {
        "ip_config_name": ip_config_name,
        "nic_name": network_interface_name,
        "resource_group": resource_group_name,
        "pool_id": backend_address_pool
    }
    if load_balancer_name:
        return LBPoolAdd(cli_ctx=cmd.cli_ctx)(command_args=arguments)
    if application_gateway_name:
        return AGPoolAdd(cli_ctx=cmd.cli_ctx)(command_args=arguments)


def remove_nic_ip_config_address_pool(cmd, resource_group_name, network_interface_name, ip_config_name,
                                      backend_address_pool, load_balancer_name=None, application_gateway_name=None):

    class LBPoolRemove(_LBPool.Remove):
        def _handler(self, command_args):
            lro_poller = super()._handler(command_args)
            lro_poller._result_callback = self._output
            return lro_poller

        def _output(self, *args, **kwargs):
            result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
            return result["ipConfigurations"][0]

    class AGPoolRemove(_AGPool.Remove):
        def _handler(self, command_args):
            lro_poller = super()._handler(command_args)
            lro_poller._result_callback = self._output
            return lro_poller

        def _output(self, *args, **kwargs):
            result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
            return result["ipConfigurations"][0]

    arguments = {
        "ip_config_name": ip_config_name,
        "nic_name": network_interface_name,
        "resource_group": resource_group_name,
        "pool_id": backend_address_pool
    }
    if load_balancer_name:
        return LBPoolRemove(cli_ctx=cmd.cli_ctx)(command_args=arguments)
    if application_gateway_name:
        return AGPoolRemove(cli_ctx=cmd.cli_ctx)(command_args=arguments)


_NICIPConfigNAT = import_aaz_by_profile("network.nic.ip_config.inbound_nat_rule")


class NICIPConfigNATAdd(_NICIPConfigNAT.Add):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lb_name = AAZStrArg(
            options=["--lb-name"],
            help="Name of the load balancer",
        )
        args_schema.inbound_nat_rule._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                     "/loadBalancers/{lb_name}/inboundNatRules/{}",
        )
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result["ipConfigurations"][0]


class NICIPConfigNATRemove(_NICIPConfigNAT.Remove):

    def _handler(self, command_args):
        lro_poller = super()._handler(command_args)
        lro_poller._result_callback = self._output
        return lro_poller

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lb_name = AAZStrArg(
            options=["--lb-name"],
            help="Name of the load balancer",
        )
        args_schema.inbound_nat_rule._fmt = AAZResourceIdArgFormat(
            template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/Microsoft.Network"
                     "/loadBalancers/{lb_name}/inboundNatRules/{}",
        )
        return args_schema

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result["ipConfigurations"][0]
